import argparse
import os
import toml

from jenesis.config import ConfigurationError
from jenesis.network import fetchai_testnet_config, fetchai_localnode_config
from jenesis.config import Deployment
from jenesis.contracts.detect import detect_contracts


def run(args: argparse.Namespace):

    project_path = os.getcwd()

    # check that we are actually running the command from the project root
    if not os.path.exists(os.path.join(project_path, "jenesis.toml")):
        # pylint: disable=all
        print("Please run command from project root")
        return

    data = toml.load("jenesis.toml")

    if args.network == "fetchai-testnet":
        net_config = fetchai_testnet_config()
    elif args.network == "fetchai-localnode":
        net_config = fetchai_localnode_config()
    else:
        raise ConfigurationError("Network name not recognized")

    network = {"name": "fetchai-testnet"}
    network.update(vars(net_config))

    # detect contract source code and add placeholders for key contract data
    contracts = detect_contracts(project_path) or []

    contract_cfgs = {contract.name: Deployment(contract,
        args.network, "", {arg: "" for arg in contract.init_args()},
        None, None, None, None,
    ) for contract in contracts}


    data["profile"][args.profile] = {
        "network": network,
        "contracts": {name: vars(cfg) for (name, cfg) in contract_cfgs.items()}
    }

    output_file_name = "jenesis.toml"
    with open(output_file_name, "w") as toml_file:
        toml.dump(data, toml_file)


def add_profile_command(parser):
    profile_cmd = parser.add_parser("profile")
    profile_cmd.add_argument("profile", default="testing", help="The profile to create"
    )
    profile_cmd.add_argument(
        "-n", "--network", default="fetchai-testnet", help="Network to use"
    )
    profile_cmd.set_defaults(handler=run)


import argparse
import os
import shutil
import subprocess
from tempfile import mkdtemp
from jenesis.contracts.detect import detect_contracts

from jenesis.config import Config
import toml

TEMPLATE_GIT_URL = "git@github.com:fetchai/jenesis-templates.git"

def add_contract_command(parser):

    add_contract_cmd = parser.add_parser("contract")
    add_contract_cmd.add_argument("template", help="The name of the template to use")
    add_contract_cmd.add_argument("name", help="The name of contract to create")
    add_contract_cmd.add_argument(
        "-b", "--branch", help="The name of the branch that should be used"
    )
    add_contract_cmd.set_defaults(handler=run_add_contract)


def run_add_contract(args: argparse.Namespace):
    template = args.template
    name = args.name
    branch = args.branch

    project_root = os.path.abspath(os.getcwd())
    contract_root = os.path.join(project_root, "contracts", name)

    # check that we are actually running the command from the project root
    if not os.path.exists(os.path.join(project_root, "jenesis.toml")):
        # pylint: disable=all
        print("Please run command from project root")
        return False

    cfg = Config.load(project_root)

    # check to see if the contract already exists
    if os.path.exists(contract_root):
        print(f'Contract "{name}" already exists')
        return False

    # create the temporary clone folder
    temp_clone_path = mkdtemp(prefix="jenesis-", suffix="-tmpl")

    # clone the templates folder out in the temporary file
    print("Downloading template...")
    cmd = ["git", "clone", "--single-branch"]
    if branch is not None:
        cmd += ["--branch", branch]
    cmd += [TEMPLATE_GIT_URL, "."]
    with open(os.devnull, "w", encoding="utf8") as null_file:
        subprocess.check_call(
            cmd, stdout=null_file, stderr=subprocess.STDOUT, cwd=temp_clone_path
        )

    # find the target contract
    contract_template_path = os.path.join(temp_clone_path, "contracts", template)
    if not os.path.isdir(contract_template_path):
        print(f"Unknown template {template}")
        return False
    print("Downloading template...complete")

    # process all the files as part of the template
    print("Rendering template...")
    for root, _, files in os.walk(contract_template_path):
        for filename in files:
            file_path = os.path.join(root, filename)
            rel_path = os.path.relpath(file_path, contract_template_path)

            output_filepath = os.path.join(contract_root, rel_path)
            os.makedirs(os.path.dirname(output_filepath), exist_ok=True)
            with open(file_path, "r", encoding="utf8") as input_file:
                with open(output_filepath, "w", encoding="utf8") as output_file:
                    contents = input_file.read()

                    # replace the templating parameters here
                    contents = contents.replace("<<NAME>>", name)

                    output_file.write(contents)
    print("Rendering template...complete")

    # clean up the temporary folder
    shutil.rmtree(temp_clone_path)

    contracts = detect_contracts(project_root)

    selected_contract = ""
    for contract in contracts:
        if contract.name == name:
            selected_contract = contract
            continue

    if selected_contract == "":
        print('Contract not found in project')
        return
               
    for profile in cfg.profiles.keys():
        network_name = cfg.profiles[profile].network.name
        Config.update_project(os.getcwd(), profile, network_name, selected_contract)

    return True



import argparse
import os

from cosmpy.aerial.client import LedgerClient
from ptpython import embed
from jenesis.config import Config
from jenesis.contracts.detect import detect_contracts
from jenesis.contracts.monkey import MonkeyContract
from jenesis.contracts.observer import DeploymentUpdater
from jenesis.network import run_local_node


def load_config(args: argparse.Namespace) -> dict:
    project_path = os.getcwd()

    # check that we are actually running the command from the project root
    if not os.path.exists(os.path.join(project_path, "jenesis.toml")):
        # pylint: disable=all
        raise RuntimeError("Please run command from project root or create project first")

    cfg = Config.load(project_path)
    contracts = detect_contracts(project_path)

    contract_instances = {}

    if args.profile is not None and args.profile not in cfg.profiles:
        print(f'Invalid profile name. Expected one of {",".join(cfg.profiles.keys())}')
        return

    profile_name = args.profile or cfg.get_default_profile()

    selected_profile = cfg.profiles.get(profile_name)
    if selected_profile is not None:
        if selected_profile.network.is_local:
            run_local_node(selected_profile.network)

        # build the ledger client
        client = LedgerClient(selected_profile.network)

        print('Detecting contracts...')

        for contract in contracts:
            print("C", contract)

            # skip contracts that we have not compiled
            if contract.digest() is None:
                continue

            # select the metadata for the contract
            selected_contract = selected_profile.deployments.get(contract.name)
            if selected_contract is not None:
                address = selected_contract.address
                code_id = selected_contract.code_id
            else:
                code_id = None
                address = None

            monkey = MonkeyContract(
                contract.binary_path,
                client,
                address=address,
                code_id=code_id,
                observer=DeploymentUpdater(
                    cfg,
                    project_path,
                    profile_name,
                    contract.name,
                ),
                init_args=selected_contract.init,
            )

            contract_instances[contract.name] = monkey

        print('Detecting contracts...complete')

    shell_globals = {}
    shell_globals["cfg"] = cfg
    shell_globals["project_path"] = project_path
    shell_globals["profile"] = profile_name
    shell_globals["contracts"] = {contract.name: contract for contract in contracts}
    for (name, instance) in contract_instances.items():
        shell_globals[name] = instance

    return shell_globals


def run(args: argparse.Namespace):
    shell_globals = load_config(args)
    shell_globals.update(globals())
    embed(shell_globals, vi_mode=False, history_filename=".shell_history")


def add_shell_command(parser):
    shell_cmd = parser.add_parser("shell")
    shell_cmd.add_argument(
        "-p", "--profile", default=None, help="The profile to use"
    )
    shell_cmd.set_defaults(handler=run)
